@@ -1,7 +1,7 @@
 /*
     GNOME Commander - A GNOME based file manager
     Copyright (C) 2001-2006 Marcus Bjurman
-    Copyright (C) 2007-2009 Piotr Eljasiak
+    Copyright (C) 2007-2010 Piotr Eljasiak
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -199,11 +199,16 @@ struct FileFormatData
     gchar *fname;
     gchar *fext;
 
+    static gchar empty_string[];
+
     FileFormatData(GnomeCmdFile *f, gboolean tree_size);
     ~FileFormatData();
 };
 
 
+gchar FileFormatData::empty_string[] = "";
+
+
 inline FileFormatData::FileFormatData(GnomeCmdFile *f, gboolean tree_size)
 {
     // If the user wants a character instead of icon for filetype set it now
@@ -240,10 +245,21 @@ inline FileFormatData::FileFormatData(GnomeCmdFile *f, gboolean tree_size)
     text[GnomeCmdFileList::COLUMN_DIR]   = dpath;
     text[GnomeCmdFileList::COLUMN_SIZE]  = tree_size ? (gchar *) gnome_cmd_file_get_tree_size_as_str (f) :
                                                        (gchar *) gnome_cmd_file_get_size (f);
-    text[GnomeCmdFileList::COLUMN_DATE]  = (gchar *) gnome_cmd_file_get_mdate (f, FALSE);
-    text[GnomeCmdFileList::COLUMN_PERM]  = (gchar *) gnome_cmd_file_get_perm (f);
-    text[GnomeCmdFileList::COLUMN_OWNER] = (gchar *) gnome_cmd_file_get_owner (f);
-    text[GnomeCmdFileList::COLUMN_GROUP] = (gchar *) gnome_cmd_file_get_group (f);
+
+    if (f->info->type != GNOME_VFS_FILE_TYPE_DIRECTORY || !f->is_dotdot)
+    {
+        text[GnomeCmdFileList::COLUMN_DATE]  = (gchar *) gnome_cmd_file_get_mdate (f, FALSE);
+        text[GnomeCmdFileList::COLUMN_PERM]  = (gchar *) gnome_cmd_file_get_perm (f);
+        text[GnomeCmdFileList::COLUMN_OWNER] = (gchar *) gnome_cmd_file_get_owner (f);
+        text[GnomeCmdFileList::COLUMN_GROUP] = (gchar *) gnome_cmd_file_get_group (f);
+    }
+    else
+    {
+        text[GnomeCmdFileList::COLUMN_DATE]  = empty_string;
+        text[GnomeCmdFileList::COLUMN_PERM]  = empty_string;
+        text[GnomeCmdFileList::COLUMN_OWNER] = empty_string;
+        text[GnomeCmdFileList::COLUMN_GROUP] = empty_string;
+    }
 }
 
 
@@ -322,7 +338,7 @@ static void select_file (GnomeCmdFileList *fl, GnomeCmdFile *f)
     g_return_if_fail (f != NULL);
     g_return_if_fail (f->info != NULL);
 
-    if (strcmp (f->info->name, "..") == 0)
+    if (f->is_dotdot)
         return;
 
     gint row = fl->get_row_from_file(f);
@@ -775,12 +791,24 @@ inline gint my_intcmp (gint i1, gint i2, gboolean raising)
 }
 
 
+inline gint my_filesizecmp (GnomeVFSFileSize i1, GnomeVFSFileSize i2, gboolean raising)
+{
+    if (i1 > i2)
+        return raising ? -1 : 1;
+
+    if (i2 > i1)
+        return raising ? 1 : -1;
+
+    return 0;
+}
+
+
 static gint sort_by_name (GnomeCmdFile *f1, GnomeCmdFile *f2, GnomeCmdFileList *fl)
 {
-    if (strcmp (f1->info->name, "..") == 0)
+    if (f1->is_dotdot)
         return -1;
 
-    if (strcmp (f2->info->name, "..") == 0)
+    if (f2->is_dotdot)
         return 1;
 
     if (f1->info->type > f2->info->type)
@@ -797,10 +825,10 @@ static gint sort_by_name (GnomeCmdFile *f1, GnomeCmdFile *f2, GnomeCmdFileList *
 
 static gint sort_by_ext (GnomeCmdFile *f1, GnomeCmdFile *f2, GnomeCmdFileList *fl)
 {
-    if (strcmp (f1->info->name, "..") == 0)
+    if (f1->is_dotdot)
         return -1;
 
-    if (strcmp (f2->info->name, "..") == 0)
+    if (f2->is_dotdot)
         return 1;
 
     if (f1->info->type > f2->info->type)
@@ -827,10 +855,10 @@ static gint sort_by_ext (GnomeCmdFile *f1, GnomeCmdFile *f2, GnomeCmdFileList *f
 
 static gint sort_by_dir (GnomeCmdFile *f1, GnomeCmdFile *f2, GnomeCmdFileList *fl)
 {
-    if (strcmp (f1->info->name, "..") == 0)
+    if (f1->is_dotdot)
         return -1;
 
-    if (strcmp (f2->info->name, "..") == 0)
+    if (f2->is_dotdot)
         return 1;
 
     if (f1->info->type > f2->info->type)
@@ -860,10 +888,10 @@ static gint sort_by_dir (GnomeCmdFile *f1, GnomeCmdFile *f2, GnomeCmdFileList *f
 
 static gint sort_by_size (GnomeCmdFile *f1, GnomeCmdFile *f2, GnomeCmdFileList *fl)
 {
-    if (strcmp (f1->info->name, "..") == 0)
+    if (f1->is_dotdot)
         return -1;
 
-    if (strcmp (f2->info->name, "..") == 0)
+    if (f2->is_dotdot)
         return 1;
 
     gboolean raising = fl->priv->sort_raising[fl->priv->current_col];
@@ -873,7 +901,7 @@ static gint sort_by_size (GnomeCmdFile *f1, GnomeCmdFile *f2, GnomeCmdFileList *
 
     if (!ret)
     {
-        ret = my_intcmp (f1->info->size, f2->info->size, raising);
+        ret = my_filesizecmp (f1->info->size, f2->info->size, raising);
         if (!ret)
             ret = my_strcmp (gnome_cmd_file_get_collation_fname (f1), gnome_cmd_file_get_collation_fname (f2), file_raising);
     }
@@ -883,10 +911,10 @@ static gint sort_by_size (GnomeCmdFile *f1, GnomeCmdFile *f2, GnomeCmdFileList *
 
 static gint sort_by_perm (GnomeCmdFile *f1, GnomeCmdFile *f2, GnomeCmdFileList *fl)
 {
-    if (strcmp (f1->info->name, "..") == 0)
+    if (f1->is_dotdot)
         return -1;
 
-    if (strcmp (f2->info->name, "..") == 0)
+    if (f2->is_dotdot)
         return 1;
 
     gboolean raising = fl->priv->sort_raising[fl->priv->current_col];
@@ -905,10 +933,10 @@ static gint sort_by_perm (GnomeCmdFile *f1, GnomeCmdFile *f2, GnomeCmdFileList *
 
 static gint sort_by_date (GnomeCmdFile *f1, GnomeCmdFile *f2, GnomeCmdFileList *fl)
 {
-    if (strcmp (f1->info->name, "..") == 0)
+    if (f1->is_dotdot)
         return -1;
 
-    if (strcmp (f2->info->name, "..") == 0)
+    if (f2->is_dotdot)
         return 1;
 
     gboolean raising = fl->priv->sort_raising[fl->priv->current_col];
@@ -927,10 +955,10 @@ static gint sort_by_date (GnomeCmdFile *f1, GnomeCmdFile *f2, GnomeCmdFileList *
 
 static gint sort_by_owner (GnomeCmdFile *f1, GnomeCmdFile *f2, GnomeCmdFileList *fl)
 {
-    if (strcmp (f1->info->name, "..") == 0)
+    if (f1->is_dotdot)
         return -1;
 
-    if (strcmp (f2->info->name, "..") == 0)
+    if (f2->is_dotdot)
         return 1;
 
     gboolean raising = fl->priv->sort_raising[fl->priv->current_col];
@@ -949,10 +977,10 @@ static gint sort_by_owner (GnomeCmdFile *f1, GnomeCmdFile *f2, GnomeCmdFileList
 
 static gint sort_by_group (GnomeCmdFile *f1, GnomeCmdFile *f2, GnomeCmdFileList *fl)
 {
-    if (strcmp (f1->info->name, "..") == 0)
+    if (f1->is_dotdot)
         return -1;
 
-    if (strcmp (f2->info->name, "..") == 0)
+    if (f2->is_dotdot)
         return 1;
 
     gboolean raising = fl->priv->sort_raising[fl->priv->current_col];
@@ -1121,7 +1149,7 @@ static void on_file_clicked (GnomeCmdFileList *fl, GnomeCmdFile *f, GdkEventButt
             }
             else
                 if (event->button == 3)
-                    if (strcmp (f->info->name, "..") != 0)
+                    if (!f->is_dotdot)
                     {
                         if (gnome_cmd_data.right_mouse_button_mode == GnomeCmdData::RIGHT_BUTTON_SELECTS)
                         {
@@ -1524,9 +1552,13 @@ void GnomeCmdFileList::update_file(GnomeCmdFile *f)
         gtk_clist_set_text (*this, row, i, data.text[i]);
 
     if (gnome_cmd_data.layout != GNOME_CMD_LAYOUT_TEXT)
-        gtk_clist_set_pixmap (*this, row, 0,
-                              gnome_cmd_file_get_type_pixmap (f),
-                              gnome_cmd_file_get_type_mask (f));
+    {
+        GdkPixmap *pixmap;
+        GdkBitmap *mask;
+
+        if (gnome_cmd_file_get_type_pixmap_and_mask (f, &pixmap, &mask))
+            gtk_clist_set_pixmap (*this, row, 0, pixmap, mask);
+    }
 }
 
 
@@ -1694,7 +1726,7 @@ void GnomeCmdFileList::focus_file(const gchar *focus_file, gboolean scroll_to_fi
 
 void GnomeCmdFileList::select_row(gint row)
 {
-    focus_file_at_row (this, row==-1 ? priv->cur_file : row);
+    focus_file_at_row (this, row==-1 ? 0 : row);
 }
 
 
@@ -2218,7 +2250,7 @@ gboolean GnomeCmdFileList::file_is_wanted(GnomeCmdFile *f)
 
     if (strcmp (info->name, ".") == 0)
         return FALSE;
-    if (strcmp (info->name, "..") == 0)
+    if (f->is_dotdot)
         return FALSE;
     if (gnome_cmd_data.hide_type(info->type))
         return FALSE;
