@@ -1,6 +1,7 @@
 /*
     GNOME Commander - A GNOME based file manager
     Copyright (C) 2001-2006 Marcus Bjurman
+    Copyright (C) 2007-2010 Piotr Eljasiak
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,8 +23,6 @@
 
 #include "gnome-cmd-file.h"
 
-G_BEGIN_DECLS
-
 #define GNOME_CMD_MAKE_COPY_DIALOG(obj) \
     GTK_CHECK_CAST (obj, gnome_cmd_make_copy_dialog_get_type (), GnomeCmdMakeCopyDialog)
 #define GNOME_CMD_MAKE_COPY_DIALOG_CLASS(klass) \
@@ -32,30 +31,24 @@ G_BEGIN_DECLS
     GTK_CHECK_TYPE (obj, gnome_cmd_make_copy_dialog_get_type ())
 
 
-typedef struct _GnomeCmdMakeCopyDialog GnomeCmdMakeCopyDialog;
-typedef struct _GnomeCmdMakeCopyDialogPrivate GnomeCmdMakeCopyDialogPrivate;
-typedef struct _GnomeCmdMakeCopyDialogClass GnomeCmdMakeCopyDialogClass;
+struct GnomeCmdMakeCopyDialogPrivate;
 
 
-struct _GnomeCmdMakeCopyDialog
+struct GnomeCmdMakeCopyDialog
 {
     GnomeCmdStringDialog parent;
     GnomeCmdMakeCopyDialogPrivate *priv;
 };
 
 
-struct _GnomeCmdMakeCopyDialogClass
+struct GnomeCmdMakeCopyDialogClass
 {
     GnomeCmdStringDialogClass parent_class;
 };
 
 
-GtkWidget*
-gnome_cmd_make_copy_dialog_new (GnomeCmdFile *finfo, GnomeCmdDir *dir);
+GtkWidget *gnome_cmd_make_copy_dialog_new (GnomeCmdFile *f, GnomeCmdDir *dir);
 
-GtkType
-gnome_cmd_make_copy_dialog_get_type (void);
-
-G_END_DECLS
+GtkType gnome_cmd_make_copy_dialog_get_type ();
 
 #endif // __GNOME_CMD_MAKE_COPY_DIALOG_H__
