@@ -1,7 +1,7 @@
 /*
     GNOME Commander - A GNOME based file manager
     Copyright (C) 2001-2006 Marcus Bjurman
-    Copyright (C) 2007-2009 Piotr Eljasiak
+    Copyright (C) 2007-2010 Piotr Eljasiak
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -432,19 +432,19 @@ static GtkWidget *create_layout_tab (GtkWidget *parent)
     GtkWidget *frame, *hbox, *vbox, *cat;
     GtkWidget *entry, *spin, *scale, *table, *label, *fpicker, *btn;
     GtkWidget *lm_optmenu, *cm_optmenu, *fe_optmenu, *check;
-    gchar *ext_modes[] = {
+    const gchar *ext_modes[] = {
         _("With file name"),
         _("In separate column"),
         _("In both columns"),
         NULL
     };
-    gchar *gfx_modes[] = {
+    const gchar *gfx_modes[] = {
         _("No icons"),
         _("File type icons"),
         _("MIME icons"),
         NULL
     };
-    gchar *color_modes[GNOME_CMD_NUM_COLOR_MODES+1] = {
+    const gchar *color_modes[GNOME_CMD_NUM_COLOR_MODES+1] = {
         _("Respect theme colors"),
         _("Modern"),
         _("Fusion"),
@@ -1026,10 +1026,7 @@ static void on_add_app_dialog_ok (GtkButton *button, GtkWidget *dialog)
     add_app_to_list (GTK_CLIST (clist), app);
     gtk_widget_destroy (dialog);
 
-    g_free (name);
-    g_free (cmd);
     g_free (icon_path);
-    g_free (pattern_string);
 }
 
 
@@ -1063,10 +1060,7 @@ static void on_edit_app_dialog_ok (GtkButton *button, GtkWidget *dialog)
     update_app_in_list (GTK_CLIST (clist), app);
     gtk_widget_destroy (dialog);
 
-    g_free (name);
-    g_free (cmd);
     g_free (icon_path);
-    g_free (pattern_string);
 }
 
 
@@ -1449,9 +1443,6 @@ static void on_add_device_dialog_ok (GtkButton *button, GtkWidget *dialog)
 
     gnome_cmd_con_list_add_device (gnome_cmd_con_list_get (), dev);
 
-    g_free (alias);
-    g_free (device);
-    g_free (mountp);
     g_free (icon_path);
 }
 
@@ -1472,9 +1463,6 @@ static void on_edit_device_dialog_ok (GtkButton *button, GtkWidget *dialog)
     update_device_in_list (GTK_CLIST (clist), dev, alias, device, mountp, icon_path);
     gtk_widget_destroy (dialog);
 
-    g_free (alias);
-    g_free (device);
-    g_free (mountp);
     g_free (icon_path);
 }
 
